@@ -1,8 +1,8 @@
 ###############################################################################
 #
-#                          Copyright (c) 2011
+#                       Copyright (c) 2011-2013
 #         Manuel Lopez-Ibanez <manuel.lopez-ibanez@ulb.ac.be>
-#             Marco Chiarandini <marco@imada.sdu.dk>
+#                Marco Chiarandini <marco@imada.sdu.dk>
 #
 # This program is free software (software libre); you can redistribute
 # it and/or modify it under the terms of the GNU General Public License
@@ -51,23 +51,21 @@
 compute.eaf <- function(data, percentiles = NULL)
 {
   if (!is.numeric(data[,1]) || !is.numeric(data[,2]) )
-    stop("The first and the second column must contain numeric data for point coordinates.\n")
-  # The C code expect points within a set to be contiguous.
+    stop("The first and the second column must contain numeric data for point coordinates")
+  # The C code expects points within a set to be contiguous.
   data <- data[order(data[,3]),]
-  npoints <- aggregate(data[,1],list(data[,3]),length)$x
+  npoints <- aggregate(data[,1], list(data[,3]), length)$x
   nsets <- length(unique(data[,3]))
   if (is.null(percentiles)) {
     percentiles <- 1:nsets * 100 / nsets
   }
-  out <- .Call("compute_eaf_C",
+  return(.Call("compute_eaf_C",
                as.double(t(as.matrix(data[,1:2]))),
                2,
                as.integer(cumsum(npoints)),
                as.integer(nsets),
                as.integer(percentiles)
-               )
-
-  return(out)
+               ))
 }
 
 compute.eaf.as.list <- function(data, percentiles = NULL)
@@ -76,7 +74,7 @@ compute.eaf.as.list <- function(data, percentiles = NULL)
   return (lapply(split.data.frame(eaf, as.factor(eaf[,3])),
                  function(x) { x[,-3, drop = FALSE] }))
 }
-
+# FIXME: The default intervals should be nsets / 2
 compute.eafdiff <- function(DATA, intervals = 1)
 {
   # The C code expects points within a set to be contiguous.
@@ -84,15 +82,11 @@ compute.eafdiff <- function(DATA, intervals = 1)
   # aggregate is quite slow on large data sets because of paste()
   npoints <- aggregate(DATA[,1], list(DATA[,3]),length)$x
   nsets <- length(unique(DATA[,3]))
-  division <- nsets %/% 2
-  nsets1 <- division
-  nsets2 <- nsets - division
   DIFF <- .Call("compute_eafdiff_C",
                 as.double(t(as.matrix(DATA[,1:2]))),
                 2,
                 as.integer(cumsum(npoints)),
                 as.integer(nsets),
-                as.integer(division),
                 as.integer(intervals)
                 )
   # FIXME: Do this computation in C code.
@@ -103,11 +97,65 @@ compute.eafdiff <- function(DATA, intervals = 1)
   return(eafdiff)
 }
 
+# FIXME: The default intervals should be nsets / 2
+compute.eafdiff.polygon <- function(DATA, intervals = 1)
+{
+  # The C code expects points within a set to be contiguous.
+  DATA <- DATA[order(DATA[,3]),]
+  # aggregate is quite slow on large data sets because of paste()
+  npoints <- aggregate(DATA[,1], list(DATA[,3]),length)$x
+  nsets <- length(unique(DATA[,3]))
+  # FIXME: Ideally this would be computed by the C code, but it is hard-coded.
+  ## division <- nsets %/% 2
+  ## nsets1 <- division
+  ## nsets2 <- nsets - division
+  return(.Call("compute_eafdiff_area_C",
+               as.double(t(as.matrix(DATA[,1:2]))),
+               2,
+               as.integer(cumsum(npoints)),
+               as.integer(nsets),
+               as.integer(intervals)
+               ))
+}
+
+rm.inf <- function(x, xmax)
+{
+  x[is.infinite(x)] <- xmax
+  return(x)
+}
+
+
+# FIXME: Accept ...
+max.finite <- function (x)
+{
+  x <- as.vector(x)
+  x <- x[is.finite(x)]
+  if (length(x)) return(max(x))
+  return(NULL)
+}
+
+# FIXME: Accept ...
+min.finite <- function (x)
+{
+  x <- as.vector(x)
+  x <- x[is.finite(x)]
+  if (length(x)) return(min(x))
+  return(NULL)
+}
+
+# FIXME: Accept ...
+range.finite <- function(x)
+{
+  return(c(min.finite(x),max.finite(x)))
+}
+
 read.data.sets <- function(file, col.names)
 {
-  stopifnot(file.exists(file))
-  out <- .Call("read_data_sets",
-               as.character(file))
+  if (!file.exists(file))
+    stop("error: ", file, ": No such file or directory");
+
+  file <- normalizePath(file)
+  out <- .Call("read_data_sets", as.character(file))
   if (missing(col.names))
     col.names <- paste("V", 1L:(ncol(out)-1), sep = "")
   colnames(out) <- c(col.names, "set")
@@ -117,14 +165,13 @@ read.data.sets <- function(file, col.names)
 ## Calculate the intermediate points in order to plot a staircase-like
 ## polygon.
 ## Example: given ((1,2), (2,1)), it returns ((1,2), (2,2), (2,1)).
-points.steps <- function(x, decreasing = FALSE)
+## Input should be already in the correct order.
+points.steps <- function(x)
 {
-  x <- rbind(x, cbind(x[-1,1],x[-nrow(x),2]))
-  if (decreasing) {
-    x <- x[order(-x[,1],x[,2]),]
-  } else {
-    x <- x[order(x[,1],-x[,2]),]
-  }
+  n <- nrow(x)
+  if (n == 1) return(x)
+  x <- rbind(x, cbind(x[-1, 1], x[-n, 2]))
+  return(x[c(as.vector(outer(c(0, n), 1:(n - 1), "+")), n),])
 }
 
 sciNotation <- function(x, digits = 1) {
@@ -142,8 +189,7 @@ eafs <- function (points = NULL, sets = NULL, groups = NULL, percentiles = NULL)
 {
   # FIXME: check conditions, ncol(points) == 2, etc.
   # MARCO: done in eafplot.default, it should be enough,
-  # unless we want to export also eafs, that is, make it callable directly from users
-  # exportable functions are to be declared in NAMESPACE
+  # unless we want to export also eafs.
   points <- cbind(points, sets)
 
   if (is.null(groups)) {
@@ -181,67 +227,109 @@ eafplot.default <-
             extra.lwd = 0.5,
             extra.lty = "dashed",
             extra.col = "black",
-            ymaximise = FALSE,
-            xaxis.side = 1, yaxis.side = 2,
+            maximise = c(FALSE, FALSE),
+            xaxis.side = "below", yaxis.side = "left",
+            axes = TRUE,
             ... )
 {
   type <- match.arg (type, c("point", "area"))
-  points <- x
+  maximise <- as.logical(maximise)
+  xaxis.side <- switch(match.arg (xaxis.side, c("below", "above")),
+                       below = 1,
+                       above = 3)
+  yaxis.side <- switch(match.arg (yaxis.side, c("left", "right")),
+                       left = 2,
+                       right = 4)
 
-  if (ymaximise) points[,2] <- -points[,2]
+  matrix.maximise <- function(z) {
+    # R bug?: If z is data.frame with rownames != NULL, and
+    # maximise == (FALSE, FALSE), then -z[, which(FALSE)]
+    # gives error: Error in
+    # data.frame(value, row.names = rn, check.names = FALSE, check.rows = FALSE) : 
+    # row names supplied are of the wrong length
+    z[, which(maximise)] <- -z[, which(maximise)]
+    return(z)
+  }
+  get.extremes <- function(xlim, ylim, maximise, log) {
+    if (length(log) && log != "")
+      log <- strsplit(log, NULL)[[1L]]
+    if ("x" %in% log) xlim <- log(xlim)
+    if ("y" %in% log) ylim <- log(ylim)
+    
+    extreme1 <- ifelse(maximise[1],
+                       xlim[1] - 0.05 * diff(xlim),
+                       xlim[2] + 0.05 * diff(xlim))
+    extreme2 <- ifelse(maximise[2],
+                       ylim[1] - 0.05 * diff(ylim),
+                       ylim[2] + 0.05 * diff(ylim))
 
-  if (is.null (xlim)) xlim <- range(points[,1])
-  if (is.null (ylim)) ylim <- range(points[,2])
+    if ("x" %in% log) extreme1 <- exp(extreme1)
+    if ("y" %in% log) extreme2 <- exp(extreme2)
+    return(c(extreme1, extreme2))
+  }
+    
+  x[, which(maximise)] <- -x[, which(maximise)]
 
   if (is.null (attsurfs)) {
-    points <- as.matrix(points)
-    if (!is.matrix(points))
-      stop("'points' must be a matrix with exactly two columns")
+    x <- as.matrix(x)
+    if (!is.matrix(x))
+      stop("'x' must be a matrix with exactly two columns")
 
-    if (nrow(points) < 1L || ncol(points) != 2)
-      stop("not enough (finite) 'points' observations or not two dimensions")
+    if (nrow(x) < 1L || ncol(x) != 2)
+      stop("not enough (finite) 'x' observations or not two dimensions")
     
-    if (!is.numeric(sets) || length(sets) != nrow(points))
-      stop("'sets' must be a vector of length equal to the number of observations in 'points'")
+    if (!is.numeric(sets) || length(sets) != nrow(x))
+      stop("'sets' must be a vector of length equal to the number of observations in 'x'")
 
-    EAF <- eafs (points, sets, groups, percentiles)
+    EAF <- eafs (x, sets, groups, percentiles)
 
     # Transform EAF matrix into attsurfs list.
     if (is.null(groups)) {
       attsurfs <- lapply(split.data.frame(EAF, as.factor(EAF[,3])),
-                         function(x) { x[,-3] })
+                         function(x) { as.matrix(x[, 1:2, drop=FALSE]) })
     } else {
       attsurfs <- list()
       groups <- EAF$groups
       for (g in levels(groups)) {
         tmp <- lapply(split.data.frame(EAF[groups == g,],
                                        as.factor(EAF[groups == g, 3])),
-                      function(x) { x[,-3] })
+                      function(x) { as.matrix(x[, 1:2, drop=FALSE]) })
         attsurfs <- c(attsurfs, tmp)
       }
     }
     # FIXME: rm(EAF) to save memory ?
+    attsurfs <- lapply(attsurfs, matrix.maximise)
+  } else {
+    attsurfs <- lapply(attsurfs, function(x) { as.matrix(x[, 1:2, drop=FALSE]) })
   }
 
-  # FIXME: Make this depend on xlim, ylim.
-  extreme <- c(1.0e+20, 1.0e+20)
+  # FIXME: This seems too complicated, what is going on?
+  if (!is.null(xlim) && maximise[1]) xlim <- -xlim 
+  if (!is.null(ylim) && maximise[2]) ylim <- -ylim
 
-  # FIXME: Find a better way to handle different x-y scale and
-  # maximization/minimization.
+  if (is.null (xlim)) xlim <- range(x[,1])
+  if (is.null (ylim)) ylim <- range(x[,2])
+
+  if (maximise[1]) xlim <- range(-xlim)
+  if (maximise[2]) ylim <- range(-ylim)
+
+  best1 <- if (maximise[1]) max else min
+  best2 <- if (maximise[2]) max else min
+
+  extreme <- get.extremes(xlim, ylim, maximise, log)
+
+  # FIXME: Find a better way to handle different x-y scale.
   yscale <- 1
-  if (ymaximise) {
-    #yscale <- 60
-    yreverse <- -1
-    attsurfs <- lapply (attsurfs, function (x)
-                        { x[,2] <- yreverse * x[,2] / yscale; x })
-    ylim <- yreverse * ylim / yscale
-    extreme[2] <- yreverse * extreme[2]
-    if (log == "y") extreme[2] <- 1
-  }
+  ## if (ymaximise) {
+  ##   #yscale <- 60
+  ##   yreverse <- -1
+  ##   attsurfs <- lapply (attsurfs, function (x)
+  ##                       { x[,2] <- yreverse * x[,2] / yscale; x })
+  ##   ylim <- yreverse * ylim / yscale
+  ##   extreme[2] <- yreverse * extreme[2]
+  ##   if (log == "y") extreme[2] <- 1
+  ## }
 
-  op <- par(cex = 1.0, cex.lab = 1.1, cex.axis = 1.0
-            , lab = c(10,5,7)
-            )
   ##'lab' A numerical vector of the form 'c(x, y, len)' which modifies
   ##     the default way that axes are annotated.  The values of 'x'
   ##     and 'y' give the (approximate) number of tickmarks on the x
@@ -250,47 +338,49 @@ eafplot.default <-
   ##     parameters 'xaxp' and 'yaxp' are set when the user coordinate
   ##     system is set up, and is not consulted when axes are drawn.
   ##     'len' _is unimplemented_ in R.
-
-  plot(xlim, ylim, type="n", xlab="", ylab="",
-       xlim = xlim, ylim = ylim, log = log, axes=FALSE,
+  op <- par(cex = 1.0, cex.lab = 1.1, cex.axis = 1.0, lab = c(10,5,7))
+  on.exit(par(op))
+  
+  plot(xlim, ylim, type = "n", xlab = "", ylab = "",
+       xlim = xlim, ylim = ylim, log = log, axes = FALSE,
        panel.first = ({
-
-         at <- axTicks(1)
-         labels <- formatC(at,format="g")
-         ## tck=1 draws the vertical grid lines (grid() is seriously broken).
-         axis(xaxis.side, at=at, labels=FALSE, tck=1, col='lightgray',
-              ## Work-around for R bug:
-              lwd=0.5, lty="26") ##  This should be instead: lty='dotted', lwd=par("lwd"))
-         axis(xaxis.side, at=at, labels=labels, las = las)
-         mtext(xlab, xaxis.side, line=2.1, cex=par("cex.axis"), las=0)
-
-         at <- axTicks(2)
-         if (ymaximise) { at <- c(yscale * c(1:24)) }
-         labels <- formatC(at,format="g")
-         if (log == "y") {
-           ## Custom log axis (like gnuplot but in R is hard)
-           max.pow <- 6
-           at <- c(1, 5, 10, 50, 100, 500, 1000, 1500, 10^c(4:max.pow))
-           labels <- c(1, 5, 10, 50, 100, 500, 1000, 1500,
-                       parse(text = paste("10^", 4:max.pow, sep = "")))
-
-           #at <- c(60, 120, 180, 240, 300, 480, 600, 900, 1200, 1440)
-           #labels <- formatC(at,format="g")
-
-           ## Now do the minor ticks, at 1/10 of each power of 10 interval
-           ##at.minor <- 2:9 * rep(c(10^c(1:max.pow)) / 10, each = length(2:9))
-           at.minor <- 1:10 * rep(c(10^c(1:max.pow)) / 10, each = length(1:10))
-           axis (yaxis.side, at = at.minor, tcl = -0.25, labels = FALSE, las=las)
-           axis (yaxis.side, at = at.minor, labels = FALSE, tck=1,
+         if (axes) {
+           at <- axTicks(1)
+           labels <- formatC(at, format="g")
+           ## tck=1 draws the vertical grid lines (grid() is seriously broken).
+           axis(xaxis.side, at=at, labels=FALSE, tck=1, col='lightgray',
+                ## Work-around for R bug:
+                lwd=0.5, lty="26") ##  This should be instead: lty='dotted', lwd=par("lwd"))
+           axis(xaxis.side, at=at, labels=labels, las = las)
+           mtext(xlab, xaxis.side, line=2.1, cex=par("cex.axis"), las=0)
+           
+           at <- axTicks(2)
+           labels <- formatC(at, format="g")
+           ## if (log == "y") {
+           ##   ## Custom log axis (like gnuplot but in R is hard)
+           ##   max.pow <- 6
+           ##   at <- c(1, 5, 10, 50, 100, 500, 1000, 1500, 10^c(4:max.pow))
+           ##   labels <- c(1, 5, 10, 50, 100, 500, 1000, 1500,
+           ##               parse(text = paste("10^", 4:max.pow, sep = "")))
+             
+           ##   #at <- c(60, 120, 180, 240, 300, 480, 600, 900, 1200, 1440)
+           ##   #labels <- formatC(at,format="g")
+             
+           ##   ## Now do the minor ticks, at 1/10 of each power of 10 interval
+           ##   ##at.minor <- 2:9 * rep(c(10^c(1:max.pow)) / 10, each = length(2:9))
+           ##   at.minor <- 1:10 * rep(c(10^c(1:max.pow)) / 10, each = length(1:10))
+           ##   axis (yaxis.side, at = at.minor, tcl = -0.25, labels = FALSE, las=las)
+           ##   axis (yaxis.side, at = at.minor, labels = FALSE, tck=1,
+           ##         col='lightgray', lty='dotted', lwd=par("lwd"))
+           ## }
+           
+           ## tck=1 draws the horizontal grid lines (grid() is seriously broken).
+           axis (yaxis.side, at=at, labels=FALSE, tck=1,
                  col='lightgray', lty='dotted', lwd=par("lwd"))
+           axis (yaxis.side, at=at, labels=labels, las = las)
+           mtext(ylab, yaxis.side, line=2.75, cex=par("cex.axis"), las=0)
          }
-
-         ## tck=1 draws the horizontal grid lines (grid() is seriously broken).
-         axis (yaxis.side, at=at, labels=FALSE, tck=1,
-               col='lightgray', lty='dotted', lwd=par("lwd"))
-         axis (yaxis.side, at=at, labels=labels, las = las)
-         mtext(ylab, yaxis.side, line=2.75, cex=par("cex.axis"), las=0)
-
+         
          # FIXME: Perhaps have a function plot.eaf.lines that computes
          # several percentiles for a single algorithm and then calls
          # points() or polygon() as appropriate to add attainment
@@ -306,20 +396,18 @@ eafplot.default <-
          if (type == "area") {
            for (k in 1:length(attsurfs)) {
              poli <- points.steps(attsurfs[[k]])
-             poli <- rbind(c(min(poli[,1]), extreme[2]), poli,
-                           c(extreme[1], min(poli[,2])), extreme)
+             poli <- rbind(c(best1(poli[,1]), extreme[2]), poli,
+                           c(extreme[1], best2(poli[,2])), extreme)
              polygon(poli[,1], poli[,2], border = NA, col = col[k])
            }
          } else {
            for (k in 1:length(attsurfs)) {
-             tmp <- attsurfs[[k]][,1:2]
-             tmp <- rbind(c(min(tmp[,1]), extreme[2]), tmp[,1:2],
-                          c(extreme[1], min(tmp[,2])))
+             tmp <- attsurfs[[k]]
+             tmp <- rbind(c(best1(tmp[,1]), extreme[2]), tmp,
+                          c(extreme[1], best2(tmp[,2])))
 
-             points(tmp[,1], tmp[,2], type="p", col=col[k], pch = pch[k],
-                    cex = cex.pch)
-             points(tmp[,1], tmp[,2], type="s", col=col[k], lty = lty[k],
-                    lwd = lwd[k])
+             points(tmp, type="p", col=col[k], pch = pch[k], cex = cex.pch)
+             points(tmp, type="s", col=col[k], lty = lty[k], lwd = lwd[k])
            }
          }
        }),
@@ -346,7 +434,6 @@ eafplot.default <-
         extra.pch[i] <- 0
 
       } else if (any(is.na(extra.points[[i]][,2]))) {
-
         abline(v=extra.points[[i]][,1], lwd = extra.lwd[i], col = extra.col[i],
                lty = extra.lty[i])
         extra.pch[i] <- NA
@@ -393,63 +480,88 @@ eafplot.default <-
   }
 
   box()
-  par(op)
+  invisible()
 }
 
 
 .plot.eafdiff.side <- function (eafdiff, attsurfs = list(),
-                               col = c("#FFFFFF", "#BFBFBF","#808080","#404040","#000000"),
-                               side = stop("Argument 'side' is required"),
-                               type = "point",
-                               xlim = NULL, ylim = NULL, log = "",
-                               las = par("las"),
-                               full.eaf = FALSE,
-                               title = "",
-                               maximise = c(FALSE, FALSE),
-                               xlab = "objective 1", ylab = "objective 2",
-                               ...)
+                                col = c("#FFFFFF", "#BFBFBF","#808080","#404040","#000000"),
+                                side = stop("Argument 'side' is required"),
+                                type = "point",
+                                xlim = NULL, ylim = NULL, log = "",
+                                las = par("las"),
+                                full.eaf = FALSE,
+                                title = "",
+                                maximise = c(FALSE, FALSE),
+                                xlab = "objective 1", ylab = "objective 2",
+                                ...)
 {
   side <- match.arg (side, c("left", "right"))
   type <- match.arg (type, c("point", "area"))
   xaxis.side <- if (side == "left") 1 else 3
   yaxis.side <- if (side == "left") 2 else 4
   maximise <- as.logical(maximise)
-  
-  # Why flooring and not ceiling? If a point has value 2.05, it should
-  # be painted with color 2 rather than 3.
-  # +1 because col[1] is white.
-  eafdiff[,3] <- floor(eafdiff[,3]) + 1
-  if (length(unique(eafdiff[,3])) > length(col)) {
-    stop ("Too few colors: length(unique(eafdiff[,3])) > length(col)")
+
+  # For !full.eaf && type == "area", str(eafdiff) is a polygon:
+  ##  $  num [, 1:2]
+  ##    - attr(*, "col")= num []
+
+  # Colors are correct for !full.eaf && type == "area"
+  if (full.eaf || type == "point") {
+    # Why flooring and not ceiling? If a point has value 2.05, it should
+    # be painted with color 2 rather than 3.
+    # +1 because col[1] is white.
+    eafdiff[,3] <- floor(eafdiff[,3]) + 1
+    if (length(unique(eafdiff[,3])) > length(col)) {
+      stop ("Too few colors: length(unique(eafdiff[,3])) > length(col)")
+    }
   }
 
-  extreme <- c(1.0e+25, 1.0e+25)
-  if (!is.null(xlim)) extreme[1] <- ifelse(maximise[1],
-                                           xlim[1] - 0.05 * diff(xlim),
-                                           xlim[2] + 0.05 * diff(xlim))
-  if (!is.null(ylim)) extreme[2] <- ifelse(maximise[2],
-                                           ylim[1] - 0.05 * diff(ylim),
-                                           ylim[2] + 0.05 * diff(ylim))
+  get.extremes <- function(xlim, ylim, maximise, log) {
+    if (length(log) && log != "")
+      log <- strsplit(log, NULL)[[1L]]
+    if ("x" %in% log) xlim <- log(xlim)
+    if ("y" %in% log) ylim <- log(ylim)
+    
+    extreme1 <- ifelse(maximise[1],
+                       xlim[1] - 0.05 * diff(xlim),
+                       xlim[2] + 0.05 * diff(xlim))
+    extreme2 <- ifelse(maximise[2],
+                       ylim[1] - 0.05 * diff(ylim),
+                       ylim[2] + 0.05 * diff(ylim))
+
+    if ("x" %in% log) extreme1 <- exp(extreme1)
+    if ("y" %in% log) extreme2 <- exp(extreme2)
+    return(c(extreme1, extreme2))
+  }
+
+  extreme <- get.extremes(xlim, ylim, maximise, log)
+
   yscale <- 1
 #    yscale <- 60
-  if (yscale != 1) {
-    eafdiff[,2] <- eafdiff[,2] / yscale
-    attsurfs <- lapply (attsurfs, function (x)
-                        { x[,2] <- x[,2] / yscale; x })
-    ylim <- ylim / yscale
-    if (log == "y") extreme[2] <- 1
-  }
+  ## if (yscale != 1) {
+  ##   # This doesn't work with polygons.
+  ##   stopifnot (full.eaf || type == "point")
 
-  attsurfs <- lapply (attsurfs,
-                      function (x) { x <- rbind(c(min(x[,1]), extreme[2]),
-                                                x[,1:2], c(extreme[1], min(x[,2]))) })
+  ##   eafdiff[,2] <- eafdiff[,2] / yscale
+  ##   attsurfs <- lapply (attsurfs, function (x)
+  ##                       { x[,2] <- x[,2] / yscale; x })
+  ##   ylim <- ylim / yscale
+  ##   if (log == "y") extreme[2] <- 1
+  ## }
 
-  plot(xlim, ylim, type="n", xlab = "", ylab = "",
-       ylim = ylim, xlim = xlim, log = log, axes=FALSE,
+  best1 <- if (maximise[1]) max else min
+  best2 <- if (maximise[2]) max else min
+  
+  attsurfs <- lapply (attsurfs, function (x) {
+    x <- rbind(c(best1(x[,1]), extreme[2]), x[,1:2],
+               c(extreme[1], best2(x[,2]))) })
+
+  plot(xlim, ylim, type = "n", xlab = "", ylab = "",
+       ylim = ylim, xlim = xlim, log = log, axes = FALSE,
        panel.first = ({
-
          at <- axTicks(1)
-         labels <- formatC(at,format="g")
+         labels <- formatC(at, format="g")
          ## tck=1 draws the vertical grid lines (grid() is seriously broken).
          axis(xaxis.side, at=at, labels=FALSE, tck=1, col='lightgray',
               ## Work-around for R bug:
@@ -460,25 +572,24 @@ eafplot.default <-
                cex = par("cex") * par("cex.axis"))
 
          at <- axTicks(2)
-         #if (ymaximise) { at <- c(yscale * c(1:24)) }
-         labels <- formatC(at,format="g")
-         if (log == "y") {
-           ## Custom log axis (like gnuplot but in R is hard)
-           max.pow <- 6
-           ##at <- c(1, 5, 10, 50, 100, 500, 1000, 1500, 10^c(4:max.pow))
-           ##labels <- c(1, 5, 10, 50, 100, 500, 1000, 1500,
-           ##            parse(text = paste("10^", 4:max.pow, sep = "")))
+         labels <- formatC(at, format="g")
+         ## if (log == "y") {
+         ##   ## Custom log axis (like gnuplot but in R is hard)
+         ##   max.pow <- 6
+         ##   ##at <- c(1, 5, 10, 50, 100, 500, 1000, 1500, 10^c(4:max.pow))
+         ##   ##labels <- c(1, 5, 10, 50, 100, 500, 1000, 1500,
+         ##   ##            parse(text = paste("10^", 4:max.pow, sep = "")))
 
-           at <- c(60, 120, 180, 240, 300, 480, 600, 900, 1200, 1440)
-           labels <- formatC(at,format="g")
+         ##   at <- c(60, 120, 180, 240, 300, 480, 600, 900, 1200, 1440)
+         ##   labels <- formatC(at,format="g")
 
-           ## Now do the minor ticks, at 1/10 of each power of 10 interval
-           at.minor <- 2:9 * rep(c(10^c(1:max.pow)) / 10, each = length(2:9))
-           at.minor <- 1:10 * rep(c(10^c(1:max.pow)) / 10, each = length(1:10))
-           ##axis (yaxis.side, at = at.minor, tcl = -0.25, labels = FALSE, las=las)
-           ##axis (yaxis.side, at = at.minor, labels = FALSE, tck=1,
-           ##      col='lightgray', lty='dotted', lwd=par("lwd"))
-         }
+         ##   ## Now do the minor ticks, at 1/10 of each power of 10 interval
+         ##   at.minor <- 2:9 * rep(c(10^c(1:max.pow)) / 10, each = length(2:9))
+         ##   at.minor <- 1:10 * rep(c(10^c(1:max.pow)) / 10, each = length(1:10))
+         ##   ##axis (yaxis.side, at = at.minor, tcl = -0.25, labels = FALSE, las=las)
+         ##   ##axis (yaxis.side, at = at.minor, labels = FALSE, tck=1,
+         ##   ##      col='lightgray', lty='dotted', lwd=par("lwd"))
+         ## }
 
          ## tck=1 draws the horizontal grid lines (grid() is seriously broken).
          axis(yaxis.side, at=at, labels=FALSE, tck=1,
@@ -489,15 +600,24 @@ eafplot.default <-
 
          if (nrow(eafdiff)) {
            if (type == "area") {
-             for (i in 1:length(col)) {
-               if (full.eaf) {
+             if (full.eaf) {
+               for (i in 1:length(col)) {
                  poli <- points.steps(eafdiff[eafdiff[,3] == i, c(1,2), drop = FALSE])
-                 poli <- rbind(c(min(poli[,1]), extreme[2]), poli,
-                               c(extreme[1], min(poli[,2])), c(extreme[1], extreme[2]))
+                 poli <- rbind(c(best1(poli[,1]), extreme[2]), poli,
+                               c(extreme[1], best2(poli[,2])), extreme)
                  polygon(poli[,1], poli[,2], border = NA, col = col[i])
-               } else {
-                 stop("The combination of parameters: type=\"area\", full.eaf=FALSE is not implemented yet.")
                }
+             } else {
+               eafdiff[,1] <- rm.inf(eafdiff[,1], extreme[1])
+               eafdiff[,2] <- rm.inf(eafdiff[,2], extreme[2])
+               polycol <- attr(eafdiff, "col")
+               #print(unique(polycol))
+               #print(length(col))
+               ## The maximum value should also be painted.
+               polycol[polycol > length(col)] <- length(col)
+               #print(eafdiff)
+               #print(col[polycol])
+               polygon(eafdiff[,1], eafdiff[,2], border = NA, col = col[polycol])
              }
            } else {
              ## The maximum value should also be painted.
@@ -511,7 +631,7 @@ eafplot.default <-
 
   lty <- c("solid", "dashed")
   lwd <- c(1)
-  if (type == "area") {
+  if (type == "area" && full.eaf) {
     col <- c("black", "black", "white")
   } else {
     col <- c("black")
@@ -522,7 +642,7 @@ eafplot.default <-
   lty <- rep(lty, len = length(attsurfs))
   col <- rep(col, len = length(attsurfs))
 
-  for (k in 1:length(attsurfs)) {
+  for (k in seq_along(attsurfs)) {
     tmp <- attsurfs[[k]]
     lines(tmp[,1], tmp[,2], type = "s", lty = lty[k], lwd = lwd[k], col = col[k])
   }
@@ -539,33 +659,39 @@ eafdiffplot <-
            col = c("#FFFFFF", "#BFBFBF","#808080","#404040","#000000"),
            percentiles = c(50),
            full.eaf = FALSE,
-           type = ifelse(full.eaf, "area", "point"),
+           type = "area",
            legend.pos = ifelse(full.eaf, "bottomleft", "topright"),
            title.left = deparse(substitute(data.left)),
            title.right = deparse(substitute(data.right)),
            xlim = NULL, ylim = NULL,
            cex = par("cex"), cex.lab = par("cex.lab"), cex.axis = par("cex.axis"),
            maximise = c(FALSE, FALSE),
+           grand.lines = TRUE,
            ...)
 {
   type <- match.arg (type, c("point", "area"))
   if (length(intervals) < length(col)) {
-    stop ("Less intervals than colors: length(intervals) < length(col)")
+    stop ("Fewer intervals than colors: length(intervals) < length(col)")
   } else if (length(intervals) > length(col)) {
     stop ("More intervals than colors: length(intervals) > length(col)")
   }
   title.left <- title.left
   title.right <- title.right
   maximise <- as.logical(maximise)
+  matrix.maximise <- function(x) {
+    x[, which(maximise)] <- -x[, which(maximise)]; return(x)
+  }
 
-  if (!is.null(xlim) && maximise[1]) xlim <- -xlim 
-  if (!is.null(ylim) && maximise[2]) ylim <- -ylim
-  
   data.left[, which(maximise)] <- -data.left[, which(maximise)]
   data.right[, which(maximise)] <- -data.right[, which(maximise)]
-  
-  attsurfs.left <- compute.eaf.as.list (data.left, percentiles)
-  attsurfs.right <- compute.eaf.as.list (data.right, percentiles)
+
+  attsurfs.left <- attsurfs.right <- list()
+  if (!any(is.na(percentiles))) {
+    attsurfs.left <- compute.eaf.as.list (data.left, percentiles)
+    attsurfs.left <- lapply(attsurfs.left, matrix.maximise)
+    attsurfs.right <- compute.eaf.as.list (data.right, percentiles)
+    attsurfs.right <- lapply(attsurfs.right, matrix.maximise)
+  }
 
   # Merge the data
   nruns.left <- max(data.left[,3])
@@ -577,6 +703,7 @@ eafdiffplot <-
   grand.attsurf <- compute.eaf.as.list (data.combined, c(0, 100))
   grand.best <- grand.attsurf[["0"]]
   grand.worst <- grand.attsurf[["100"]]
+
   if (full.eaf) {
     DIFF <- list()
     if (type == "area") {
@@ -595,22 +722,36 @@ eafdiffplot <-
     DIFF$right[,3] <- DIFF$right[,3] * length(intervals) / 100
     #remove(data.left,data.right,data.combined) # Free memory?
   } else {
-    #remove(data.left,data.right) # Free memory?
-    DIFF <- compute.eafdiff (data.combined, intervals = length(intervals))
-    #remove(data.combined) # Free memory?
+    if (type == "area") {
+      DIFF <- compute.eafdiff.polygon (data.combined, intervals = length(intervals))
+    } else if (type == "point") {
+      #remove(data.left,data.right) # Free memory?
+      DIFF <- compute.eafdiff (data.combined, intervals = length(intervals))
+      #remove(data.combined) # Free memory?
+    }
   }
 
+  if (!is.null(xlim) && maximise[1]) xlim <- -xlim 
+  if (!is.null(ylim) && maximise[2]) ylim <- -ylim
+
   if (is.null(xlim)) {
     xlim <- range(c(grand.best[,1], grand.worst[,1],
-                    DIFF$left[,1], DIFF$right[,1]))
+                    range.finite(DIFF$left[,1]), range.finite(DIFF$right[,1])))
   }
 
   if (is.null(ylim)) {
     ylim <- range(c(grand.best[,2], grand.worst[,2],
-                    DIFF$left[,2], DIFF$right[,2]))
+                    range.finite(DIFF$left[,2]), range.finite(DIFF$right[,2])))
   }
+  if (maximise[1]) xlim <- range(-xlim)
+  if (maximise[2]) ylim <- range(-ylim)
+  
+  grand.best[, which(maximise)] <- -grand.best[, which(maximise)]
+  grand.worst[, which(maximise)] <- -grand.worst[, which(maximise)]
+  DIFF$left[, which(maximise)] <- -DIFF$left[, which(maximise)]
+  DIFF$right[, which(maximise)] <- -DIFF$right[, which(maximise)]
 
-  layout(matrix(c(1,1,2,2), 2, 2, byrow = FALSE))
+  layout(matrix(1:2, ncol = 2))
   bottommar <- 5
   topmar   <- 4
   leftmar  <- 4
@@ -621,18 +762,16 @@ eafdiffplot <-
             , mar = c(bottommar, leftmar, topmar, 0)
             , lab = c(10,5,7)
             , las = 0
+            , pty = 's' # Force it to be square
             )
-
-  if (maximise[1]) xlim <- range(-xlim)
-  if (maximise[2]) ylim <- range(-ylim)
-
-  DIFF$left[, which(maximise)] <- - DIFF$left[, which(maximise)]
-  DIFF$right[, which(maximise)] <- - DIFF$right[, which(maximise)]
-
-  attsurfs <- c(list(grand.best), attsurfs.left, list(grand.worst))
-  attsurfs <- lapply (attsurfs, function (x)
-                      { x[, which(maximise)] <- -x[, which(maximise)]; x })
-
+  on.exit(par(op))
+  
+  if (grand.lines) {
+    attsurfs <- c(list(grand.best), attsurfs.left, list(grand.worst))
+  } else {
+    attsurfs <- attsurfs.left
+  }
+  
   .plot.eafdiff.side (DIFF$left,
                      attsurfs = attsurfs,
                      col = col,
@@ -649,10 +788,12 @@ eafdiffplot <-
 
   par(mar = c(bottommar, 0, topmar, rightmar))
 
-  attsurfs <- c(list(grand.best), attsurfs.right, list(grand.worst))
-  attsurfs <- lapply (attsurfs, function (x)
-                      { x[, which(maximise)] <- -x[, which(maximise)]; x })
-
+  if (grand.lines) {
+    attsurfs <- c(list(grand.best), attsurfs.right, list(grand.worst))
+  } else {
+    attsurfs <- attsurfs.right
+  }
+  
   .plot.eafdiff.side (DIFF$right,
                       attsurfs = attsurfs,
                       col = col,
@@ -660,7 +801,7 @@ eafdiffplot <-
                       title = title.right,
                       xlim = xlim, ylim = ylim,
                       side = "right", maximise = maximise, ...)
-  par(op)
+  invisible()
 }
 
 ### Local Variables:
